@@ -14,7 +14,8 @@
         jruby-config (RubyInstanceConfig.)
         gem-home     (get-in config [:jruby-puppet :gem-home])
         env          (doto (HashMap. (.getEnvironment jruby-config))
-                       (.put "GEM_HOME" gem-home))
+                       (.put "GEM_HOME" gem-home)
+                       (.put "JARS_NO_REQUIRE" "true"))
         jruby-home   (.getJRubyHome jruby-config)
         load-path    (->> (get-in config [:os-settings :ruby-load-path])
                           (cons jruby-puppet/ruby-code-dir)
