@@ -15,10 +15,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Definitions
 
-(def default-pool-size
-  "The default size of each JRuby pool."
-  (+ 2 (ks/num-cpus)))
-
 (def pool-queue-type
   "The Java datastructure type used to store JRubyPuppet instances which are
   free to be borrowed."
@@ -68,21 +64,39 @@
         if not specified, will use the puppet default.
 
     * :max-active-instances - The maximum number of JRubyPuppet instances that
-        will be pooled. If not specified, the system's
-        number of CPUs+2 will be used.
+        will be pooled.
 
     * :http-client-ssl-protocols - A list of legal SSL protocols that may be used
         when https client requests are made.
 
     * :http-client-cipher-suites - A list of legal SSL cipher suites that may
-        be used when https client requests are made."
-  {:ruby-load-path                                  [schema/Str]
-   :gem-home                                        schema/Str
-   (schema/optional-key :master-conf-dir)           schema/Str
-   (schema/optional-key :master-var-dir)            schema/Str
-   (schema/optional-key :max-active-instances)      schema/Int
-   (schema/optional-key :http-client-ssl-protocols) [schema/Str]
-   (schema/optional-key :http-client-cipher-suites) [schema/Str]})
+        be used when https client requests are made.
+
+    * :http-client-connect-timeout-milliseconds - The amount of time, in
+        milliseconds, that an outbound HTTP connection will wait to connect
+        before giving up. Defaults to 2 minutes if not set. If 0, the timeout is
+        infinite and if negative, the value is undefined in the application and
+        governed by the system default behavior.
+
+    * :http-client-idle-timeout-milliseconds - The amount of time, in
+        milliseconds, that an outbound HTTP connection will wait for data to be
+        available after a request is sent before closing the socket. Defaults to
+        2 minutes. If 0, the timeout is infinite and if negative, the value is
+        undefined by the application and is governed by the default system
+        behavior.
+
+    * :borrow-timeout - The timeout when borrowing instances from the JRuby Pool
+        in milliseconds. Defaults to 1200000."
+  {:ruby-load-path                                                 [schema/Str]
+   :gem-home                                                       schema/Str
+   (schema/optional-key :master-conf-dir)                          schema/Str
+   (schema/optional-key :master-var-dir)                           schema/Str
+   (schema/optional-key :max-active-instances)                     schema/Int
+   (schema/optional-key :http-client-ssl-protocols)                [schema/Str]
+   (schema/optional-key :http-client-cipher-suites)                [schema/Str]
+   (schema/optional-key :http-client-connect-timeout-milliseconds) schema/Int
+   (schema/optional-key :http-client-idle-timeout-milliseconds)    schema/Int
+   (schema/optional-key :borrow-timeout)                           schema/Int})
 
 (def PoolState
   "A map that describes all attributes of a particular JRubyPuppet pool."
@@ -129,6 +143,13 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Private
 
+(defn default-pool-size
+  "Calculate the default size of the JRuby pool, based on the number of cpus."
+  [num-cpus]
+  (->> (- num-cpus 1)
+       (max 1)
+       (min 4)))
+
 (defn prep-scripting-container
   [scripting-container ruby-load-path gem-home]
   (doto scripting-container
@@ -136,7 +157,9 @@
                          (map fs/absolute-path ruby-load-path)))
     (.setCompatVersion (CompatVersion/RUBY1_9))
     (.setCompileMode RubyInstanceConfig$CompileMode/OFF)
-    (.setEnvironment (merge {"GEM_HOME" gem-home} jruby-puppet-env))))
+    (.setEnvironment (merge {"GEM_HOME" gem-home
+                             "JARS_NO_REQUIRE" "true"}
+                            jruby-puppet-env))))
 
 (defn empty-scripting-container
   "Creates a clean instance of `org.jruby.embed.ScriptingContainer` with no code loaded."
@@ -172,7 +195,9 @@
    config   :- JRubyPuppetConfig
    profiler :- (schema/maybe PuppetProfiler)]
   (let [{:keys [ruby-load-path gem-home master-conf-dir master-var-dir
-                http-client-ssl-protocols http-client-cipher-suites]} config]
+                http-client-ssl-protocols http-client-cipher-suites
+                http-client-connect-timeout-milliseconds
+                http-client-idle-timeout-milliseconds]} config]
     (when-not ruby-load-path
       (throw (Exception.
                "JRuby service missing config value 'ruby-load-path'")))
@@ -185,13 +210,17 @@
         (.put puppet-config "confdir" (fs/absolute-path master-conf-dir)))
       (when master-var-dir
         (.put puppet-config "vardir" (fs/absolute-path master-var-dir)))
-        
+
       (when http-client-ssl-protocols
         (.put puppet-server-config "ssl_protocols" (into-array String http-client-ssl-protocols)))
       (when http-client-cipher-suites
         (.put puppet-server-config "cipher_suites" (into-array String http-client-cipher-suites)))
       (.put puppet-server-config "profiler" profiler)
       (.put puppet-server-config "environment_registry" env-registry)
+      (.put puppet-server-config "http_connect_timeout_milliseconds"
+            http-client-connect-timeout-milliseconds)
+      (.put puppet-server-config "http_idle_timeout_milliseconds"
+            http-client-idle-timeout-milliseconds)
 
       (let [instance (map->JRubyPuppetInstance
                        {:pool                 pool
@@ -244,7 +273,15 @@
   create-pool-from-config :- PoolState
   "Create a new PoolData based on the config input."
   [{size :max-active-instances} :- JRubyPuppetConfig]
-  (let [size (or size default-pool-size)]
+  (let [size (if size
+               size
+               (let [default-size (default-pool-size (ks/num-cpus))]
+                 (log/warn (str "No configuration value found for jruby-puppet "
+                                "max-active-instances; using default value of "
+                                default-size ".  Please consider setting this "
+                                "value explicitly in the jruby-puppet section "
+                                "of your Puppet Server config files."))
+                 default-size))]
     {:pool         (instantiate-free-pool size)
      :size         size}))
 
@@ -314,4 +351,4 @@
   return-to-pool
   "Return a borrowed pool instance to its free pool."
   [instance :- JRubyPuppetInstanceOrRetry]
-  (.put (:pool instance) instance))
\ No newline at end of file
+  (.put (:pool instance) instance))
