@@ -7,6 +7,24 @@
             [puppetlabs.services.protocols.jruby-puppet :as jruby]
             [puppetlabs.services.jruby.jruby-puppet-core :as jruby-core]))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Constants
+
+(def default-borrow-timeout
+  "Default timeout when borrowing instances from the JRuby pool in
+   milliseconds. Current value is 1200000ms, or 20 minutes."
+  1200000)
+
+(def default-http-connect-timeout
+  "The default number of milliseconds that the client will wait for a connection
+  to be established. Currently set to 2 minutes."
+  (* 2 60 1000))
+
+(def default-http-socket-timeout
+  "The default number of milliseconds that the client will allow for no data to
+  be available on the socket. Currently set to 20 minutes."
+  (* 20 60 1000))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
@@ -25,30 +43,34 @@
                               (assoc :http-client-ssl-protocols
                                      (get-in-config [:http-client :ssl-protocols]))
                               (assoc :http-client-cipher-suites
-                                     (get-in-config [:http-client :cipher-suites])))
+                                     (get-in-config [:http-client :cipher-suites]))
+                              (assoc :http-client-connect-timeout-milliseconds
+                                     (get-in-config [:http-client :connect-timeout-milliseconds]
+                                                    default-http-connect-timeout))
+                              (assoc :http-client-idle-timeout-milliseconds
+                                     (get-in-config [:http-client :idle-timeout-milliseconds]
+                                                    default-http-socket-timeout)))
           service-id        (tk-services/service-id this)
           agent-shutdown-fn (partial shutdown-on-error service-id)
-          pool-agent  (jruby-agents/pool-agent agent-shutdown-fn)
-          profiler          (get-profiler)]
+          pool-agent        (jruby-agents/pool-agent agent-shutdown-fn)
+          profiler          (get-profiler)
+          borrow-timeout    (get-in-config [:jruby-puppet :borrow-timeout]
+                                           default-borrow-timeout)]
       (core/verify-config-found! config)
       (log/info "Initializing the JRuby service")
       (let [pool-context (core/create-pool-context config profiler)]
         (jruby-agents/send-prime-pool! pool-context pool-agent)
         (-> context
             (assoc :pool-context pool-context)
-            (assoc :pool-agent pool-agent)))))
+            (assoc :pool-agent pool-agent)
+            (assoc :borrow-timeout borrow-timeout)))))
 
   (borrow-instance
     [this]
-    (let [pool-context (:pool-context (tk-services/service-context this))
-          pool         (core/get-pool pool-context)]
-      (core/borrow-from-pool pool)))
-
-  (borrow-instance
-    [this timeout]
-    (let [pool-context (:pool-context (tk-services/service-context this))
-          pool         (core/get-pool pool-context)]
-      (core/borrow-from-pool-with-timeout pool timeout)))
+    (let [pool-context   (:pool-context (tk-services/service-context this))
+          pool           (core/get-pool pool-context)
+          borrow-timeout (:borrow-timeout (tk-services/service-context this))]
+      (core/borrow-from-pool-with-timeout pool borrow-timeout)))
 
   (return-instance
     [this jruby-instance]
@@ -59,7 +81,7 @@
     (let [pool-context (:pool-context (tk-services/service-context this))
           pool         (core/get-pool pool-context)]
       (core/free-instance-count pool)))
-  
+
   (mark-all-environments-expired!
     [this]
     (let [pool-context (:pool-context (tk-services/service-context this))]
@@ -79,9 +101,15 @@
     (with-jruby-puppet
       jruby-puppet
       jruby-service
-      (do-something-with-a-jruby-puppet-instance jruby-puppet)))"
+      (do-something-with-a-jruby-puppet-instance jruby-puppet)))
+
+  Will throw an IllegalStateException if borrowing an instance of
+  JRubyPuppet times out."
   [jruby-puppet jruby-service & body]
   `(loop [pool-instance# (jruby/borrow-instance ~jruby-service)]
+     (if (nil? pool-instance#)
+       (throw (IllegalStateException.
+                "Error: Attempt to borrow a JRuby instance from the pool timed out")))
      (if (core/retry-poison-pill? pool-instance#)
        (do
          (jruby-core/return-to-pool pool-instance#)
